<?php
namespace Elementor\Core\Common\Modules\EventTracker\Data;

use Elementor\Core\Common\Modules\EventTracker\DB as Events_DB_Manager;
use Elementor\Plugin;
use WP_REST_Server;
use Elementor\Data\V2\Base\Controller as Controller_Base;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

class Controller extends Controller_Base {

	public function get_name() {
		return 'send-event';
	}

	public function register_endpoints() {
		$this->index_endpoint->register_items_route( \WP_REST_Server::CREATABLE, [
			'event_data' => [
				'description' => 'All the recorded event data in JSON format',
				'type' => 'object',
				'required' => true,
			],
		] );
	}

	/**
	 * Get Permissions Callback
	 *
	 * This endpoint should only accept POST requests, and currently we only track site administrator actions.
	 *
	 * @since 3.6.0
	 *
	 * @param \WP_REST_Request $request
	 * @return bool
	 */
	public function g<?php
namespace Elementor\Core\Common\Modules\EventTracker;

use Elementor\Core\Base\Module as BaseModule;
use Elementor\Core\Common\Modules\EventTracker\Data\Controller;
use Elementor\Plugin;
use Elementor\Tracker;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Event Tracker Module Class
 *
 * @since 3.6.0
 */
class Module extends BaseModule {

	public function get_name() {
		return 'event-tracker';
	}

	/**
	 * Get init settings.
	 *
	 * @since 3.6.0
	 * @access protected
	 *
	 * @return array
	 */
	protected function get_init_settings() {
		return [
			'isUserDataShared' => Tracker::is_allow_track(),
		];
	}

	public function __construct() {
		// Initialize Events Database Table
		$this->add_component( 'events-db', new DB() );

		// Handle User Data Deletion/Export requests.
		new Personal_Data();

		Plugin::$instance->data_manager_v2->register_controller( new Controller() );
	}
}
